@@ -31,14 +31,18 @@
 #include <gudev/gudev.h>
 
 #include "up-backend.h"
+#include "up-backend-linux-private.h"
 #include "up-daemon.h"
 #include "up-device.h"
 
+#include "sysfs-utils.h"
+
 #include "up-device-supply.h"
 #include "up-device-csr.h"
 #include "up-device-unifying.h"
 #include "up-device-wup.h"
 #include "up-device-hid.h"
+#include "up-device-bluez.h"
 #include "up-input.h"
 #include "up-config.h"
 #ifdef HAVE_IDEVICE
@@ -65,6 +69,10 @@ struct UpBackendPrivate
 	GDBusProxy		*logind_proxy;
 	guint                    logind_sleep_id;
 	int                      logind_inhibitor_fd;
+
+	/* BlueZ */
+	guint			 bluez_watch_id;
+	GDBusObjectManager	*bluez_client;
 };
 
 enum {
@@ -80,9 +88,6 @@ G_DEFINE_TYPE (UpBackend, up_backend, G_TYPE_OBJECT)
 static gboolean up_backend_device_add (UpBackend *backend, GUdevDevice *native);
 static void up_backend_device_remove (UpBackend *backend, GUdevDevice *native);
 
-/**
- * up_backend_device_new:
- **/
 static UpDevice *
 up_backend_device_new (UpBackend *backend, GUdevDevice *native)
 {
@@ -100,10 +105,9 @@ up_backend_device_new (UpBackend *backend, GUdevDevice *native)
 		ret = up_device_coldplug (device, backend->priv->daemon, G_OBJECT (native));
 		if (ret)
 			goto out;
-		g_object_unref (device);
 
 		/* no valid power supply object */
-		device = NULL;
+		g_clear_object (&device);
 
 	} else if (g_strcmp0 (subsys, "hid") == 0) {
 
@@ -112,10 +116,8 @@ up_backend_device_new (UpBackend *backend, GUdevDevice *native)
 		ret = up_device_coldplug (device, backend->priv->daemon, G_OBJECT (native));
 		if (ret)
 			goto out;
-		g_object_unref (device);
-
 		/* no valid power supply object */
-		device = NULL;
+		g_clear_object (&device);
 
 	} else if (g_strcmp0 (subsys, "tty") == 0) {
 
@@ -124,10 +126,9 @@ up_backend_device_new (UpBackend *backend, GUdevDevice *native)
 		ret = up_device_coldplug (device, backend->priv->daemon, G_OBJECT (native));
 		if (ret)
 			goto out;
-		g_object_unref (device);
 
 		/* no valid TTY object */
-		device = NULL;
+		g_clear_object (&device);
 
 	} else if (g_strcmp0 (subsys, "usb") == 0 || g_strcmp0 (subsys, "usbmisc") == 0) {
 
@@ -152,10 +153,9 @@ up_backend_device_new (UpBackend *backend, GUdevDevice *native)
 		ret = up_device_coldplug (device, backend->priv->daemon, G_OBJECT (native));
 		if (ret)
 			goto out;
-		g_object_unref (device);
 
 		/* no valid USB object */
-		device = NULL;
+		g_clear_object (&device);
 
 	} else if (g_strcmp0 (subsys, "input") == 0) {
 
@@ -181,9 +181,6 @@ out:
 	return device;
 }
 
-/**
- * up_backend_device_changed:
- **/
 static void
 up_backend_device_changed (UpBackend *backend, GUdevDevice *native)
 {
@@ -207,13 +204,9 @@ up_backend_device_changed (UpBackend *backend, GUdevDevice *native)
 		goto out;
 	}
 out:
-	if (object != NULL)
-		g_object_unref (object);
+	g_clear_object (&object);
 }
 
-/**
- * up_backend_device_add:
- **/
 static gboolean
 up_backend_device_add (UpBackend *backend, GUdevDevice *native)
 {
@@ -241,14 +234,10 @@ up_backend_device_add (UpBackend *backend, GUdevDevice *native)
 	/* emit */
 	g_signal_emit (backend, signals[SIGNAL_DEVICE_ADDED], 0, native, device);
 out:
-	if (object != NULL)
-		g_object_unref (object);
+	g_clear_object (&object);
 	return ret;
 }
 
-/**
- * up_backend_device_remove:
- **/
 static void
 up_backend_device_remove (UpBackend *backend, GUdevDevice *native)
 {
@@ -268,13 +257,9 @@ up_backend_device_remove (UpBackend *backend, GUdevDevice *native)
 	g_signal_emit (backend, signals[SIGNAL_DEVICE_REMOVED], 0, native, device);
 
 out:
-	if (object != NULL)
-		g_object_unref (object);
+	g_clear_object (&object);
 }
 
-/**
- * up_backend_uevent_signal_handler_cb:
- **/
 static void
 up_backend_uevent_signal_handler_cb (GUdevClient *client, const gchar *action,
 				      GUdevDevice *device, gpointer user_data)
@@ -291,10 +276,216 @@ up_backend_uevent_signal_handler_cb (GUdevClient *client, const gchar *action,
 		g_debug ("SYSFS change %s", g_udev_device_get_sysfs_path (device));
 		up_backend_device_changed (backend, device);
 	} else {
-		g_warning ("unhandled action '%s' on %s", action, g_udev_device_get_sysfs_path (device));
+		g_debug ("unhandled action '%s' on %s", action, g_udev_device_get_sysfs_path (device));
 	}
 }
 
+static gpointer
+is_macbook (gpointer data)
+{
+	char *product;
+	gboolean ret = FALSE;
+
+	product = sysfs_get_string ("/sys/devices/virtual/dmi/id/", "product_name");
+	if (product == NULL)
+		return GINT_TO_POINTER(ret);
+	ret = g_str_has_prefix (product, "MacBook");
+	g_free (product);
+	return GINT_TO_POINTER(ret);
+}
+
+gboolean
+up_backend_needs_poll_after_uevent (void)
+{
+	static GOnce dmi_once = G_ONCE_INIT;
+	g_once (&dmi_once, is_macbook, NULL);
+	return GPOINTER_TO_INT(dmi_once.retval);
+}
+
+static gboolean
+is_battery_iface_proxy (GDBusProxy *interface_proxy)
+{
+	const char *iface;
+
+	iface = g_dbus_proxy_get_interface_name (interface_proxy);
+	return g_str_equal (iface, "org.bluez.Battery1");
+}
+
+static gboolean
+has_battery_iface (GDBusObject *object)
+{
+	GDBusInterface *iface;
+
+	iface = g_dbus_object_get_interface (object, "org.bluez.Battery1");
+	if (!iface)
+		return FALSE;
+	g_object_unref (iface);
+	return TRUE;
+}
+
+static void
+bluez_proxies_changed (GDBusObjectManagerClient *manager,
+		       GDBusObjectProxy         *object_proxy,
+		       GDBusProxy               *interface_proxy,
+		       GVariant                 *changed_properties,
+		       GStrv                     invalidated_properties,
+		       gpointer                  user_data)
+{
+	UpBackend *backend = user_data;
+	GObject *object;
+	UpDeviceBluez *bluez;
+
+	if (!is_battery_iface_proxy (interface_proxy))
+		return;
+
+	object = up_device_list_lookup (backend->priv->device_list, G_OBJECT (object_proxy));
+	if (!object)
+		return;
+
+	bluez = UP_DEVICE_BLUEZ (object);
+	up_device_bluez_update (bluez, changed_properties);
+	g_object_unref (object);
+}
+
+static void
+bluez_interface_removed (GDBusObjectManager *manager,
+			 GDBusObject        *bus_object,
+			 GDBusInterface     *interface,
+			 gpointer            user_data)
+{
+	UpBackend *backend = user_data;
+	GObject *object;
+
+	/* It might be another iface on another device that got removed */
+	if (has_battery_iface (bus_object))
+		return;
+
+	object = up_device_list_lookup (backend->priv->device_list, G_OBJECT (bus_object));
+	if (!object)
+		return;
+
+	g_debug ("emitting device-removed: %s", g_dbus_object_get_object_path (bus_object));
+	g_signal_emit (backend, signals[SIGNAL_DEVICE_REMOVED], 0, bus_object, UP_DEVICE (object));
+
+	g_object_unref (object);
+}
+
+static void
+bluez_interface_added (GDBusObjectManager *manager,
+		       GDBusObject        *bus_object,
+		       GDBusInterface     *interface,
+		       gpointer            user_data)
+{
+	UpBackend *backend = user_data;
+	UpDevice *device;
+	GObject *object;
+	gboolean ret;
+
+	if (!has_battery_iface (bus_object))
+		return;
+
+	object = up_device_list_lookup (backend->priv->device_list, G_OBJECT (bus_object));
+	if (object != NULL) {
+		g_object_unref (object);
+		return;
+	}
+
+	device = UP_DEVICE (up_device_bluez_new ());
+	ret = up_device_coldplug (device, backend->priv->daemon, G_OBJECT (bus_object));
+	if (!ret) {
+		g_object_unref (device);
+		return;
+	}
+
+	g_debug ("emitting device-added: %s", g_dbus_object_get_object_path (bus_object));
+	g_signal_emit (backend, signals[SIGNAL_DEVICE_ADDED], 0, bus_object, device);
+}
+
+static void
+bluez_appeared (GDBusConnection *connection,
+		const gchar     *name,
+		const gchar     *name_owner,
+		gpointer         user_data)
+{
+	UpBackend *backend = user_data;
+	GError *error = NULL;
+	GList *objects, *l;
+
+	g_assert (backend->priv->bluez_client == NULL);
+
+	backend->priv->bluez_client = g_dbus_object_manager_client_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+										     G_DBUS_OBJECT_MANAGER_CLIENT_FLAGS_DO_NOT_AUTO_START,
+										     "org.bluez",
+										     "/",
+										     NULL, NULL, NULL,
+										     NULL, &error);
+	if (!backend->priv->bluez_client) {
+		g_warning ("Failed to create object manager for BlueZ: %s",
+			   error->message);
+		g_error_free (error);
+		return;
+	}
+
+	g_debug ("BlueZ appeared");
+
+	g_signal_connect (backend->priv->bluez_client, "interface-proxy-properties-changed",
+			  G_CALLBACK (bluez_proxies_changed), backend);
+	g_signal_connect (backend->priv->bluez_client, "interface-removed",
+			  G_CALLBACK (bluez_interface_removed), backend);
+	g_signal_connect (backend->priv->bluez_client, "interface-added",
+			  G_CALLBACK (bluez_interface_added), backend);
+
+	objects = g_dbus_object_manager_get_objects (backend->priv->bluez_client);
+	for (l = objects; l != NULL; l = l->next) {
+		GDBusObject *object = l->data;
+		GList *interfaces, *k;
+
+		interfaces = g_dbus_object_get_interfaces (object);
+
+		for (k = interfaces; k != NULL; k = k->next) {
+			GDBusInterface *iface = k->data;
+
+			bluez_interface_added (backend->priv->bluez_client,
+					       object,
+					       iface,
+					       backend);
+			g_object_unref (iface);
+		}
+		g_list_free (interfaces);
+		g_object_unref (object);
+	}
+	g_list_free (objects);
+}
+
+static void
+bluez_vanished (GDBusConnection *connection,
+		const gchar     *name,
+		gpointer         user_data)
+{
+	UpBackend *backend = user_data;
+	GPtrArray *array;
+	guint i;
+
+	g_debug ("BlueZ disappeared");
+
+	array = up_device_list_get_array (backend->priv->device_list);
+
+	for (i = 0; i < array->len; i++) {
+		UpDevice *device = UP_DEVICE (g_ptr_array_index (array, i));
+		if (UP_IS_DEVICE_BLUEZ (device)) {
+			GDBusObject *object;
+
+			object = G_DBUS_OBJECT (up_device_get_native (device));
+			g_debug ("emitting device-removed: %s", g_dbus_object_get_object_path (object));
+			g_signal_emit (backend, signals[SIGNAL_DEVICE_REMOVED], 0, object, UP_DEVICE (object));
+		}
+	}
+
+	g_ptr_array_unref (array);
+
+	g_clear_object (&backend->priv->bluez_client);
+}
+
 /**
  * up_backend_coldplug:
  * @backend: The %UpBackend class instance
@@ -335,6 +526,14 @@ up_backend_coldplug (UpBackend *backend, UpDaemon *daemon)
 		g_list_free_full (devices, (GDestroyNotify) g_object_unref);
 	}
 
+	backend->priv->bluez_watch_id = g_bus_watch_name (G_BUS_TYPE_SYSTEM,
+							  "org.bluez",
+							  G_BUS_NAME_WATCHER_FLAGS_NONE,
+							  bluez_appeared,
+							  bluez_vanished,
+							  backend,
+							  NULL);
+
 	return TRUE;
 }
 
@@ -348,21 +547,17 @@ up_backend_coldplug (UpBackend *backend, UpDaemon *daemon)
 void
 up_backend_unplug (UpBackend *backend)
 {
-	if (backend->priv->gudev_client != NULL) {
-		g_object_unref (backend->priv->gudev_client);
-		backend->priv->gudev_client = NULL;
-	}
-	if (backend->priv->device_list != NULL) {
-		g_object_unref (backend->priv->device_list);
-		backend->priv->device_list = NULL;
-	}
+	g_clear_object (&backend->priv->gudev_client);
+	g_clear_object (&backend->priv->device_list);
 	/* set in init, clear the list to remove reference to UpDaemon */
 	if (backend->priv->managed_devices != NULL)
 		up_device_list_clear (backend->priv->managed_devices, FALSE);
-	if (backend->priv->daemon != NULL) {
-		g_object_unref (backend->priv->daemon);
-		backend->priv->daemon = NULL;
+	g_clear_object (&backend->priv->daemon);
+	if (backend->priv->bluez_watch_id > 0) {
+		g_bus_unwatch_name (backend->priv->bluez_watch_id);
+		backend->priv->bluez_watch_id = 0;
 	}
+	g_clear_object (&backend->priv->bluez_client);
 }
 
 static gboolean
@@ -498,7 +693,9 @@ up_backend_inhibitor_lock_take (UpBackend *backend)
 							NULL,
 							&error);
 	if (out == NULL) {
-		g_warning ("Could not acquire inhibitor lock: %s", error->message);
+		g_warning ("Could not acquire inhibitor lock: %s",
+			   error ? error->message : "Unknown reason");
+		g_clear_error (&error);
 		return;
 	}
 
@@ -590,10 +787,6 @@ up_backend_prepare_for_sleep (GDBusConnection *connection,
 }
 
 
-/**
- * up_backend_class_init:
- * @klass: The UpBackendClass
- **/
 static void
 up_backend_class_init (UpBackendClass *klass)
 {
@@ -616,9 +809,6 @@ up_backend_class_init (UpBackendClass *klass)
 	g_type_class_add_private (klass, sizeof (UpBackendPrivate));
 }
 
-/**
- * up_backend_init:
- **/
 static void
 up_backend_init (UpBackend *backend)
 {
@@ -654,9 +844,6 @@ up_backend_init (UpBackend *backend)
 	up_backend_inhibitor_lock_take (backend);
 }
 
-/**
- * up_backend_finalize:
- **/
 static void
 up_backend_finalize (GObject *object)
 {
@@ -667,13 +854,16 @@ up_backend_finalize (GObject *object)
 
 	backend = UP_BACKEND (object);
 
-	g_object_unref (backend->priv->config);
-	if (backend->priv->daemon != NULL)
-		g_object_unref (backend->priv->daemon);
-	if (backend->priv->device_list != NULL)
-		g_object_unref (backend->priv->device_list);
-	if (backend->priv->gudev_client != NULL)
-		g_object_unref (backend->priv->gudev_client);
+	if (backend->priv->bluez_watch_id > 0) {
+		g_bus_unwatch_name (backend->priv->bluez_watch_id);
+		backend->priv->bluez_watch_id = 0;
+	}
+	g_clear_object (&backend->priv->bluez_client);
+
+	g_clear_object (&backend->priv->config);
+	g_clear_object (&backend->priv->daemon);
+	g_clear_object (&backend->priv->device_list);
+	g_clear_object (&backend->priv->gudev_client);
 
 	bus = g_dbus_proxy_get_connection (backend->priv->logind_proxy);
 	g_dbus_connection_signal_unsubscribe (bus,
