@@ -60,7 +60,6 @@ typedef enum
 GQuark		 up_daemon_error_quark		(void);
 GType		 up_daemon_get_type		(void);
 UpDaemon	*up_daemon_new			(void);
-void		 up_daemon_test		(gpointer	 user_data);
 
 /* private */
 guint		 up_daemon_get_number_devices_of_type (UpDaemon	*daemon,
@@ -87,6 +86,8 @@ UpDeviceLevel	 up_daemon_compute_warning_level(UpDaemon		*daemon,
 void		 up_daemon_start_poll		(GObject		*object,
 						 GSourceFunc		 callback);
 void		 up_daemon_stop_poll		(GObject		*object);
+void             up_daemon_pause_poll           (UpDaemon               *daemon);
+void             up_daemon_resume_poll          (UpDaemon               *daemon);
 
 G_END_DECLS
 
